@@ -1,12 +1,13 @@
 # --------------------------------------------------------------- Imports ---------------------------------------------------------------- #
 
 # System
-from typing import Optional, List, Dict, Union, Any
+from typing import Optional, List, Dict, Union, Tuple, Any
 
 # Pip
 from jsoncodable import JSONCodable
 
 from talib import abstract
+import pandas
 import numpy
 
 # Local
@@ -23,16 +24,19 @@ ALL = __ALL_METHODS_KEY
 
 # Supports 'ALL' as type
 def get_stats(
-    data: Dict[str, numpy.ndarray],
-    types: Union[List[FunctionName], str] = __ALL_METHODS_KEY,
-    timeperiod: int = 30,
+    data: Union[Dict[str, numpy.ndarray], Dict[str, pandas.Series], pandas.DataFrame],
+    functions: Dict[Union[FunctionName, str], Tuple[Optional[Tuple], Optional[Dict[str, Any]]]] = {__ALL_METHODS_KEY: (None, None)},
+    timeperiod: Optional[int] = None,
     use_builtin_types: bool = False,
-    use_objects: bool = False
-) -> Optional[Union[Dict[str, Any], object]]:
+    use_objects: bool = False,
+) -> Optional[Union[Dict[str, Union[List[numpy.ndarray], numpy.ndarray, pandas.DataFrame]], object]]:
     """Creates TA data for the selected types of analysises
 
     Args:
-        data (Dict[str, numpy.ndarray]): The data to analyze. Should be provided in the following format:
+        data (Union[Dict[str, numpy.ndarray], Dict[str, pandas.Series], pandas.DataFrame]):
+        The data to analyze. Either a dict of numpy.ndarray or pandas.Series, or a pandas.DataFrame.
+        If a pandas.DataFrame is provided, the output is returned as a Dict[str, pandas.DataFrame] with named output columns for each.
+        For dict, it should look like:
         {
             'open': numpy.ndarray,
             'high': numpy.ndarray,
@@ -41,10 +45,21 @@ def get_stats(
             'volume': numpy.ndarray,
         }
 
-        types ([Union[List[FunctionName], str]]): The analysis types, you want to perform.
-                                                  Defaults to ALL.
+        functions (Dict[Union[FunctionName, str], Tuple[Optional[Tuple], Optional[Dict[str, Any]]]]):
+        The analysis types, you want to perform Each with args and kwargs.
+        Example: {
+            FunctionName.STOCH: (
+                (5, 3, 0, 3, 0,),
+                {
+                    prices: ['high', 'low', 'open']
+                }
+            )
+        }
+        This would equal to STOCH(inputs, 5, 3, 0, 3, 0, prices=['high', 'low', 'open'])
+        Defaults to {ALL: (None, None)}.
 
-        timeperiod (int, optional): Time period to calculate analysis with.
+        timeperiod (int, optional): Time period to calculate analysis with. Will e set only for those functions,
+                                    which do not specify timeperiod explicitaly.
                                     Defaults to 30.
 
         use_builtin_types (bool, optional): returns python list and int/float instead of numpy types.
@@ -69,22 +84,39 @@ def get_stats(
     Returns:
         Optional[Union[Dict[str, Any], object]]: Returns dictionary or objects, based on the provided values for 'use_builtin_types' and 'use_objects'
     """
-    if isinstance(types, str):
-        if types.upper() == __ALL_METHODS_KEY:
-            types = [function_name for function_name in FunctionName]
-        else:
-            return None
 
-    func_names = [function_name.value for function_name in types]
+    if isinstance(data, pandas.DataFrame):
+        use_builtin_types = False
+        use_objects = False
+
+    if __ALL_METHODS_KEY in functions.keys():
+        params = functions[__ALL_METHODS_KEY]
+
+        for func_name_enum in FunctionName:
+            if func_name_enum not in functions:
+                functions[func_name_enum] = params
+
+        del functions[__ALL_METHODS_KEY]
+
     res = {}
 
-    for func_name in func_names:
+    for func_name_enum, args_kwargs in functions.items():
+        func_name = func_name_enum.value
         func = abstract.Function(func_name)
 
-        try:
-            _res = func(data, timeperiod=timeperiod)
+        args, kwargs = args_kwargs
+        k_timeperiod = 'timeperiod'
 
-            if use_builtin_types or use_objects:
+        args = (data,) if args is None else (data,) + args
+        kwargs = kwargs or { k_timeperiod: timeperiod }
+
+        if k_timeperiod not in kwargs:
+            kwargs[k_timeperiod] = timeperiod
+
+        try:
+            _res = func(*args, **kwargs)
+
+            if (use_builtin_types or use_objects):
                 _res = __convert_to_builtin(_res)
 
             res[func_name] = _res
